@@ -22,6 +22,7 @@ import (
 	"sort"
 	"strconv"
 	"testing"
+	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"go.uber.org/zap/zapcore"
@@ -31,20 +32,24 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
+	network "knative.dev/networking/pkg"
+	"knative.dev/networking/pkg/apis/networking"
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/logging"
 	"knative.dev/pkg/metrics"
-	_ "knative.dev/pkg/metrics/testing"
 	"knative.dev/pkg/ptr"
 	"knative.dev/pkg/system"
-	_ "knative.dev/pkg/system/testing"
 	tracingconfig "knative.dev/pkg/tracing/config"
-	"knative.dev/serving/pkg/apis/networking"
+	apicfg "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
-	autoscalerconfig "knative.dev/serving/pkg/autoscaler/config"
+	"knative.dev/serving/pkg/autoscaler/config/autoscalerconfig"
 	"knative.dev/serving/pkg/deployment"
-	"knative.dev/serving/pkg/network"
+	"knative.dev/serving/pkg/queue"
+	"knative.dev/serving/pkg/reconciler/revision/config"
+
+	_ "knative.dev/pkg/metrics/testing"
+	_ "knative.dev/pkg/system/testing"
 )
 
 var (
@@ -56,13 +61,34 @@ var (
 		},
 	}
 
+	containers = []corev1.Container{{
+		Name:           servingContainerName,
+		Image:          "busybox",
+		ReadinessProbe: withTCPReadinessProbe(v1.DefaultUserPort),
+	}}
+
 	// The default CM values.
-	logConfig        logging.Config
-	traceConfig      tracingconfig.Config
-	obsConfig        metrics.ObservabilityConfig
-	asConfig         autoscalerconfig.Config
+	asConfig = autoscalerconfig.Config{
+		InitialScale:          1,
+		AllowZeroInitialScale: false,
+	}
 	deploymentConfig = deployment.Config{
-		ProgressDeadline: deployment.ProgressDeadlineDefault,
+		ProgressDeadline: 5678 * time.Second,
+	}
+	logConfig   logging.Config
+	obsConfig   metrics.ObservabilityConfig
+	traceConfig tracingconfig.Config
+	defaults, _ = apicfg.NewDefaultsConfigFromMap(nil)
+	revCfg      = config.Config{
+		Config: &apicfg.Config{
+			Autoscaler: &asConfig,
+			Defaults:   defaults,
+		},
+		Deployment:    &deploymentConfig,
+		Logging:       &logConfig,
+		Network:       &network.Config{},
+		Observability: &obsConfig,
+		Tracing:       &traceConfig,
 	}
 )
 
@@ -73,79 +99,73 @@ func TestMakeQueueContainer(t *testing.T) {
 		name string
 		rev  *v1.Revision
 		lc   logging.Config
+		nc   network.Config
 		oc   metrics.ObservabilityConfig
-		cc   deployment.Config
+		dc   deployment.Config
 		want corev1.Container
 	}{{
-		name: "no owner no autoscaler single",
+		name: "autoscaler single",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers),
+			withContainerConcurrency(1)),
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
 				"CONTAINER_CONCURRENCY": "1",
 			})
 		}),
 	}, {
-		name: "no owner no autoscaler single",
+		name: "custom sidecar image, container port, protocol",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.Spec.PodSpec = corev1.PodSpec{
-					Containers: []corev1.Container{{
-						Name:           containerName,
-						ReadinessProbe: testProbe,
-						Ports: []corev1.ContainerPort{{
-							ContainerPort: 1955,
-							Name:          string(networking.ProtocolH2C),
-						}},
-					}},
-				}
-			}),
-		cc: deployment.Config{
+			withContainers([]corev1.Container{{
+				Name:           servingContainerName,
+				ReadinessProbe: testProbe,
+				Ports: []corev1.ContainerPort{{
+					ContainerPort: 1955,
+					Name:          string(networking.ProtocolH2C),
+				}},
+			}})),
+		dc: deployment.Config{
+			ProgressDeadline:  5678 * time.Second,
 			QueueSidecarImage: "alpine",
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Image = "alpine"
 			c.Ports = append(queueNonServingPorts, queueHTTP2Port)
+			c.ReadinessProbe.Handler.HTTPGet.Port.IntVal = queueHTTP2Port.ContainerPort
 			c.Env = env(map[string]string{
-				"USER_PORT":             "1955",
-				"QUEUE_SERVING_PORT":    "8013",
-				"CONTAINER_CONCURRENCY": "1",
+				"USER_PORT":          "1955",
+				"QUEUE_SERVING_PORT": "8013",
 			})
-		},
-		),
+		}),
 	}, {
 		name: "service name in labels",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
+			withContainers(containers),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.ObjectMeta.Labels = map[string]string{
+				revision.Labels = map[string]string{
 					serving.ServiceLabelKey: "svc",
 				}
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
 			}),
-		cc: deployment.Config{
-			QueueSidecarImage: "alpine",
-		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
 				"SERVING_SERVICE": "svc",
 			})
-			c.Image = "alpine"
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
 		}),
 	}, {
 		name: "config owner as env var, zero concurrency",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("blah", "baz",
+			withContainers(containers),
 			withContainerConcurrency(0),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
 				revision.ObjectMeta.OwnerReferences = []metav1.OwnerReference{{
 					APIVersion:         v1.SchemeGroupVersion.String(),
 					Kind:               "Configuration",
@@ -153,7 +173,6 @@ func TestMakeQueueContainer(t *testing.T) {
 					Controller:         ptr.Bool(true),
 					BlockOwnerDeletion: ptr.Bool(true),
 				}}
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
 			}),
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
@@ -162,16 +181,14 @@ func TestMakeQueueContainer(t *testing.T) {
 				"SERVING_NAMESPACE":     "baz",
 				"SERVING_REVISION":      "blah",
 			})
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
 		}),
 	}, {
 		name: "logging configuration as env var",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("this", "log",
-			withContainerConcurrency(0),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers)),
 		lc: logging.Config{
 			LoggingConfig: "The logging configuration goes here",
 			LoggingLevel: map[string]zapcore.Level{
@@ -180,82 +197,158 @@ func TestMakeQueueContainer(t *testing.T) {
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"CONTAINER_CONCURRENCY":  "0",
 				"SERVING_LOGGING_CONFIG": "The logging configuration goes here",
 				"SERVING_LOGGING_LEVEL":  "error",
 				"SERVING_NAMESPACE":      "log",
 				"SERVING_REVISION":       "this",
 			})
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
 		}),
 	}, {
 		name: "container concurrency 10",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainerConcurrency(10),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers),
+			withContainerConcurrency(10)),
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
 				"CONTAINER_CONCURRENCY": "10",
 			})
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
 		}),
 	}, {
 		name: "request log configuration as env var",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(0),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{
 			RequestLogTemplate:    "test template",
 			EnableProbeRequestLog: true,
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"CONTAINER_CONCURRENCY":            "0",
 				"SERVING_REQUEST_LOG_TEMPLATE":     "test template",
 				"SERVING_ENABLE_PROBE_REQUEST_LOG": "true",
 			})
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
+		}),
+	}, {
+		name: "disabled request log configuration as env var",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
+		oc: metrics.ObservabilityConfig{
+			RequestLogTemplate:    "test template",
+			EnableProbeRequestLog: false,
+			EnableRequestLog:      false,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"SERVING_REQUEST_LOG_TEMPLATE":     "test template",
+				"SERVING_ENABLE_REQUEST_LOG":       "false",
+				"SERVING_ENABLE_PROBE_REQUEST_LOG": "false",
+			})
 		}),
 	}, {
 		name: "request metrics backend as env var",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(0),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{
 			RequestMetricsBackend: "prometheus",
 		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"CONTAINER_CONCURRENCY":           "0",
 				"SERVING_REQUEST_METRICS_BACKEND": "prometheus",
 			})
-			c.Ports = append(queueNonServingPorts, queueHTTPPort)
 		}),
 	}, {
 		name: "enable profiling",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(0),
-			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				container(revision.Spec.GetContainer(), withTCPReadinessProbe())
-			}),
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		oc: metrics.ObservabilityConfig{EnableProfiling: true},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"CONTAINER_CONCURRENCY": "0",
-				"ENABLE_PROFILING":      "true",
+				"ENABLE_PROFILING": "true",
 			})
 			c.Ports = append(queueNonServingPorts, profilingPort, queueHTTPPort)
 		}),
+	}, {
+		name: "custom TimeoutSeconds",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
+		rev: revision("bar", "foo",
+			withContainers(containers),
+			func(revision *v1.Revision) {
+				revision.Spec.TimeoutSeconds = ptr.Int64(99)
+			},
+		),
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"REVISION_TIMEOUT_SECONDS": "99",
+			})
+		}),
+	}, {
+		name: "default resource config",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
+			QueueSidecarCPURequest: &deployment.QueueSidecarCPURequestDefault,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{})
+			c.Resources.Requests = corev1.ResourceList{
+				corev1.ResourceCPU: resource.MustParse("25m"),
+			}
+			c.Resources.Limits = nil
+		}),
+	}, {
+		name: "overridden resources",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		dc: deployment.Config{
+			QueueSidecarCPURequest:              resourcePtr(resource.MustParse("123m")),
+			QueueSidecarEphemeralStorageRequest: resourcePtr(resource.MustParse("456M")),
+			QueueSidecarMemoryLimit:             resourcePtr(resource.MustParse("789m")),
+			ProgressDeadline:                    5678 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{})
+			c.Resources.Requests = corev1.ResourceList{
+				corev1.ResourceCPU:              resource.MustParse("123m"),
+				corev1.ResourceEphemeralStorage: resource.MustParse("456M"),
+			}
+			c.Resources.Limits = corev1.ResourceList{
+				corev1.ResourceMemory: resource.MustParse("789m"),
+			}
+		}),
+	}, {
+		name: "collector address as env var",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		oc: metrics.ObservabilityConfig{
+			RequestMetricsBackend:   "opencensus",
+			MetricsCollectorAddress: "otel:55678",
+		},
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"SERVING_REQUEST_METRICS_BACKEND": "opencensus",
+				"METRICS_COLLECTOR_ADDRESS":       "otel:55678",
+			})
+		}),
 	}}
 
 	for _, test := range tests {
@@ -263,12 +356,18 @@ func TestMakeQueueContainer(t *testing.T) {
 			if len(test.rev.Spec.PodSpec.Containers) == 0 {
 				test.rev.Spec.PodSpec = corev1.PodSpec{
 					Containers: []corev1.Container{{
-						Name:           containerName,
+						Name:           servingContainerName,
 						ReadinessProbe: testProbe,
 					}},
 				}
 			}
-			got, err := makeQueueContainer(test.rev, &test.lc, &traceConfig, &test.oc, &asConfig, &test.cc)
+			cfg := &config.Config{
+				Tracing:       &traceConfig,
+				Logging:       &test.lc,
+				Observability: &test.oc,
+				Deployment:    &test.dc,
+			}
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -279,8 +378,8 @@ func TestMakeQueueContainer(t *testing.T) {
 			})
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainer (-want, +got) =", diff)
+			if got, want := *got, test.want; !cmp.Equal(got, want, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
@@ -291,23 +390,24 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 		name string
 		rev  *v1.Revision
 		want corev1.Container
+		dc   deployment.Config
 	}{{
 		name: "resources percentage in annotations",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.ObjectMeta.Labels = map[string]string{}
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "20",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
-					Name:           containerName,
+					Name:           servingContainerName,
 					ReadinessProbe: testProbe,
 					Resources: corev1.ResourceRequirements{
 						Limits: corev1.ResourceList{
-							corev1.ResourceName("memory"): resource.MustParse("2Gi"),
-							corev1.ResourceName("cpu"):    resource.MustParse("2"),
+							corev1.ResourceMemory: resource.MustParse("2Gi"),
+							corev1.ResourceCPU:    resource.MustParse("2"),
 						},
 					}},
 				}
@@ -315,28 +415,27 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{})
 			c.Resources.Limits = corev1.ResourceList{
-				corev1.ResourceName("memory"): *resource.NewMilliQuantity(429496729600, resource.BinarySI),
-				corev1.ResourceName("cpu"):    *resource.NewMilliQuantity(400, resource.BinarySI),
+				corev1.ResourceMemory: resource.MustParse("0.4Gi"),
+				corev1.ResourceCPU:    resource.MustParse("0.4"),
 			}
-			c.Image = "alpine"
 		}),
 	}, {
-		name: "resources percentage in annotations small than min allowed",
+		name: "resources percentage in annotations smaller than min allowed",
+		dc: deployment.Config{
+			ProgressDeadline: 5678 * time.Second,
+		},
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.ObjectMeta.Labels = map[string]string{}
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "0.2",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
-					Name:           containerName,
+					Name:           servingContainerName,
 					ReadinessProbe: testProbe,
 					Resources: corev1.ResourceRequirements{
 						Requests: corev1.ResourceList{
-							corev1.ResourceName("cpu"):    resource.MustParse("50m"),
-							corev1.ResourceName("memory"): resource.MustParse("128Mi"),
+							corev1.ResourceCPU:    resource.MustParse("50m"),
+							corev1.ResourceMemory: resource.MustParse("128Mi"),
 						},
 					},
 				}}
@@ -344,73 +443,73 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{})
 			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"):    resource.MustParse("25m"),
-				corev1.ResourceName("memory"): resource.MustParse("50Mi"),
+				corev1.ResourceCPU:    resource.MustParse("25m"), // clamped to boundary in resourceboundary.go
+				corev1.ResourceMemory: resource.MustParse("50Mi"),
 			}
-			c.Image = "alpine"
 		}),
 	}, {
-		name: "Invalid resources percentage in annotations",
+		name: "invalid resources percentage in annotations uses defaults",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.ObjectMeta.Labels = map[string]string{}
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "foo",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
-					Name:           containerName,
+					Name:           servingContainerName,
 					ReadinessProbe: testProbe,
 					Resources: corev1.ResourceRequirements{
 						Requests: corev1.ResourceList{
-							corev1.ResourceName("cpu"):    resource.MustParse("50m"),
-							corev1.ResourceName("memory"): resource.MustParse("128Mi"),
+							corev1.ResourceCPU:    resource.MustParse("50m"),
+							corev1.ResourceMemory: resource.MustParse("128Mi"),
 						},
 					},
 				}}
 			}),
+		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
+			QueueSidecarCPURequest: resourcePtr(resource.MustParse("25m")),
+		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{})
 			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"): resource.MustParse("25m"),
+				corev1.ResourceCPU: resource.MustParse("25m"),
 			}
-			c.Image = "alpine"
 		}),
 	}, {
 		name: "resources percentage in annotations bigger than than math.MaxInt64",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.Spec.TimeoutSeconds = ptr.Int64(45)
-				revision.ObjectMeta.Labels = map[string]string{}
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "100",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
-					Name:           containerName,
+					Name:           servingContainerName,
 					ReadinessProbe: testProbe,
 					Resources: corev1.ResourceRequirements{
 						Requests: corev1.ResourceList{
-							corev1.ResourceName("memory"): resource.MustParse("900000Pi"),
+							corev1.ResourceMemory: resource.MustParse("900000Pi"),
 						},
 					},
 				}}
 			}),
+		dc: deployment.Config{
+			ProgressDeadline:       5678 * time.Second,
+			QueueSidecarCPURequest: resourcePtr(resource.MustParse("25m")),
+		},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{})
 			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"):    resource.MustParse("25m"),
-				corev1.ResourceName("memory"): resource.MustParse("200Mi"),
+				corev1.ResourceCPU:    resource.MustParse("25m"),
+				corev1.ResourceMemory: resource.MustParse("200Mi"),
 			}
-			c.Image = "alpine"
 		}),
 	}}
 
-	cc := deployment.Config{QueueSidecarImage: "alpine"}
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			got, err := makeQueueContainer(test.rev, &logConfig, &traceConfig, &obsConfig, &asConfig, &cc)
+			cfg := (&revCfg).DeepCopy()
+			cfg.Deployment = &test.dc
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -420,20 +519,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 			})
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainerWithPercentageAnnotation (-want, +got) =", diff)
-			}
-			if test.want.Resources.Limits.Memory().Cmp(*got.Resources.Limits.Memory()) != 0 {
-				t.Errorf("Resources.Limits.Memory = %v, want: %v", got.Resources.Limits.Memory(), test.want.Resources.Limits.Memory())
-			}
-			if test.want.Resources.Requests.Cpu().Cmp(*got.Resources.Requests.Cpu()) != 0 {
-				t.Errorf("Resources.Request.CPU = %v, want: %v", got.Resources.Requests.Cpu(), test.want.Resources.Requests.Cpu())
-			}
-			if test.want.Resources.Requests.Memory().Cmp(*got.Resources.Requests.Memory()) != 0 {
-				t.Errorf("Resources.Requests.Memory = %v, want: %v", got.Resources.Requests.Memory(), test.want.Resources.Requests.Memory())
-			}
-			if test.want.Resources.Limits.Cpu().Cmp(*got.Resources.Limits.Cpu()) != 0 {
-				t.Errorf("Resources.Limits.CPU  = %v, want: %v", got.Resources.Limits.Cpu(), test.want.Resources.Limits.Cpu())
+			if got, want := *got, test.want; !cmp.Equal(got, want, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
@@ -441,12 +528,9 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 
 func TestProbeGenerationHTTPDefaults(t *testing.T) {
 	rev := revision("bar", "foo",
-		withContainerConcurrency(1),
 		func(revision *v1.Revision) {
-			revision.Spec.TimeoutSeconds = ptr.Int64(45)
-			revision.ObjectMeta.Labels = map[string]string{}
 			revision.Spec.PodSpec.Containers = []corev1.Container{{
-				Name: containerName,
+				Name: servingContainerName,
 				ReadinessProbe: &corev1.Probe{
 					Handler: corev1.Handler{
 						HTTPGet: &corev1.HTTPGetAction{
@@ -468,7 +552,7 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 				Scheme: corev1.URISchemeHTTP,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -485,28 +569,29 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		c.Env = env(map[string]string{
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
-		c.Resources.Requests = corev1.ResourceList{
-			corev1.ResourceName("cpu"): resource.MustParse("25m"),
-		}
+		c.StartupProbe = nil // User overrode periodSeconds so no exec probe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10"},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
 				},
 			},
 			PeriodSeconds:  1,
 			TimeoutSeconds: 10,
 		}
-	},
-	)
+	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &asConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
 	sortEnv(got.Env)
-	if diff := cmp.Diff(want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-		t.Error("makeQueueContainer(-want, +got) =", diff)
+	if got, want := *got, want; !cmp.Equal(got, want, quantityComparer) {
+		t.Errorf("makeQueueContainer(-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 	}
 }
 
@@ -515,12 +600,9 @@ func TestProbeGenerationHTTP(t *testing.T) {
 	const probePath = "/health"
 
 	rev := revision("bar", "foo",
-		withContainerConcurrency(1),
 		func(revision *v1.Revision) {
-			revision.Spec.TimeoutSeconds = ptr.Int64(45)
-			revision.ObjectMeta.Labels = map[string]string{}
 			revision.Spec.PodSpec.Containers = []corev1.Container{{
-				Name: containerName,
+				Name: servingContainerName,
 				Ports: []corev1.ContainerPort{{
 					ContainerPort: userPort,
 				}},
@@ -546,7 +628,7 @@ func TestProbeGenerationHTTP(t *testing.T) {
 				Scheme: corev1.URISchemeHTTPS,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -564,24 +646,29 @@ func TestProbeGenerationHTTP(t *testing.T) {
 			"USER_PORT":               strconv.Itoa(userPort),
 			"SERVING_READINESS_PROBE": string(wantProbeJSON),
 		})
+		c.StartupProbe = nil // User overrode periodSeconds so no execprobe needed.
 		c.ReadinessProbe = &corev1.Probe{
 			Handler: corev1.Handler{
-				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-period", "10"},
-				}},
+				HTTPGet: &corev1.HTTPGetAction{
+					Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+					HTTPHeaders: []corev1.HTTPHeader{{
+						Name:  network.ProbeHeaderName,
+						Value: queue.Name,
+					}},
+				},
+			},
 			PeriodSeconds:  2,
 			TimeoutSeconds: 10,
 		}
-	},
-	)
+	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &asConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
 	sortEnv(got.Env)
-	if diff := cmp.Diff(want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-		t.Error("makeQueueContainer(-want, +got) =", diff)
+	if got, want := *got, want; !cmp.Equal(got, want, quantityComparer) {
+		t.Errorf("makeQueueContainer(-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 	}
 }
 
@@ -605,11 +692,10 @@ func TestTCPProbeGeneration(t *testing.T) {
 			SuccessThreshold: 3,
 		},
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
-					Name: containerName,
+					Name: servingContainerName,
 					Ports: []corev1.ContainerPort{{
 						ContainerPort: userPort,
 					}},
@@ -624,28 +710,44 @@ func TestTCPProbeGeneration(t *testing.T) {
 			},
 		},
 		want: queueContainer(func(c *corev1.Container) {
-			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"): resource.MustParse("25m"),
+			c.StartupProbe = &corev1.Probe{
+				Handler: corev1.Handler{
+					Exec: &corev1.ExecAction{
+						Command: []string{"/ko-app/queue", "-probe-period", "1h34m38s"},
+					},
+				},
+				// StartupProbe overrides the user's parameters because the
+				// actual probing happens inside the execprobe.
+				// The timeout seconds is set to the progress deadline to match the
+				// timeout on initial deployment.
+				TimeoutSeconds:   5678,
+				PeriodSeconds:    1,
+				SuccessThreshold: 1,
+				FailureThreshold: 1,
 			}
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "0"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
-				PeriodSeconds:  1,
-				TimeoutSeconds: 10,
+				PeriodSeconds:    1,
+				TimeoutSeconds:   0,
+				SuccessThreshold: 3,
 			}
 			c.Env = env(map[string]string{"USER_PORT": strconv.Itoa(userPort)})
 		}),
 	}, {
 		name: "tcp defaults",
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
-					Name: containerName,
+					Name: servingContainerName,
 					ReadinessProbe: &corev1.Probe{
 						Handler: corev1.Handler{
 							TCPSocket: &corev1.TCPSocketAction{},
@@ -666,17 +768,20 @@ func TestTCPProbeGeneration(t *testing.T) {
 			TimeoutSeconds: 1,
 		},
 		want: queueContainer(func(c *corev1.Container) {
-			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"): resource.MustParse("25m"),
-			}
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "1"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
-				PeriodSeconds:  1,
-				TimeoutSeconds: 1,
+				PeriodSeconds: 1,
+				// Inherit Kubernetes default here rather than overriding as we need to do for exec probe.
+				TimeoutSeconds: 0,
 			}
 			c.Env = env(map[string]string{})
 		}),
@@ -696,11 +801,10 @@ func TestTCPProbeGeneration(t *testing.T) {
 			InitialDelaySeconds: 3,
 		},
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
-					Name: containerName,
+					Name: servingContainerName,
 					Ports: []corev1.ContainerPort{{
 						ContainerPort: userPort,
 					}},
@@ -718,13 +822,15 @@ func TestTCPProbeGeneration(t *testing.T) {
 			},
 		},
 		want: queueContainer(func(c *corev1.Container) {
-			c.Resources.Requests = corev1.ResourceList{
-				corev1.ResourceName("cpu"): resource.MustParse("25m"),
-			}
+			c.StartupProbe = nil // User overrode periodSeconds, no exec probe needed.
 			c.ReadinessProbe = &corev1.Probe{
 				Handler: corev1.Handler{
-					Exec: &corev1.ExecAction{
-						Command: []string{"/ko-app/queue", "-probe-period", "15"},
+					HTTPGet: &corev1.HTTPGetAction{
+						Port: intstr.FromInt(int(queueHTTPPort.ContainerPort)),
+						HTTPHeaders: []corev1.HTTPHeader{{
+							Name:  network.ProbeHeaderName,
+							Value: queue.Name,
+						}},
 					},
 				},
 				PeriodSeconds:       2,
@@ -751,42 +857,43 @@ func TestTCPProbeGeneration(t *testing.T) {
 				Value: string(wantProbeJSON),
 			})
 
-			got, err := makeQueueContainer(testRev, &logConfig, &traceConfig, &obsConfig, &asConfig, &deploymentConfig)
+			got, err := makeQueueContainer(testRev, &revCfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error")
 			}
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainer (-want, +got) =", diff)
+			if got, want := *got, test.want; !cmp.Equal(want, got, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
 }
 
 var defaultEnv = map[string]string{
-	"SERVING_NAMESPACE":                     "foo",
-	"SERVING_SERVICE":                       "",
-	"SERVING_CONFIGURATION":                 "",
-	"SERVING_REVISION":                      "bar",
-	"CONTAINER_CONCURRENCY":                 "1",
+	"CONTAINER_CONCURRENCY":                 "0",
+	"ENABLE_PROFILING":                      "false",
+	"METRICS_DOMAIN":                        metrics.Domain(),
+	"METRICS_COLLECTOR_ADDRESS":             "",
+	"QUEUE_SERVING_PORT":                    "8012",
 	"REVISION_TIMEOUT_SECONDS":              "45",
+	"SERVING_CONFIGURATION":                 "",
+	"SERVING_ENABLE_PROBE_REQUEST_LOG":      "false",
+	"SERVING_ENABLE_REQUEST_LOG":            "false",
 	"SERVING_LOGGING_CONFIG":                "",
 	"SERVING_LOGGING_LEVEL":                 "",
-	"TRACING_CONFIG_BACKEND":                "",
-	"TRACING_CONFIG_ZIPKIN_ENDPOINT":        "",
-	"TRACING_CONFIG_STACKDRIVER_PROJECT_ID": "",
-	"TRACING_CONFIG_SAMPLE_RATE":            "0",
-	"TRACING_CONFIG_DEBUG":                  "false",
+	"SERVING_NAMESPACE":                     "foo",
 	"SERVING_REQUEST_LOG_TEMPLATE":          "",
 	"SERVING_REQUEST_METRICS_BACKEND":       "",
-	"USER_PORT":                             strconv.Itoa(v1.DefaultUserPort),
+	"SERVING_REVISION":                      "bar",
+	"SERVING_SERVICE":                       "",
 	"SYSTEM_NAMESPACE":                      system.Namespace(),
-	"METRICS_DOMAIN":                        metrics.Domain(),
-	"QUEUE_SERVING_PORT":                    "8012",
-	"DOWNWARD_API_LABELS_PATH":              fmt.Sprintf("%s/%s", podInfoVolumePath, metadataLabelsPath),
-	"ENABLE_PROFILING":                      "false",
-	"SERVING_ENABLE_PROBE_REQUEST_LOG":      "false",
+	"TRACING_CONFIG_BACKEND":                "",
+	"TRACING_CONFIG_DEBUG":                  "false",
+	"TRACING_CONFIG_SAMPLE_RATE":            "0",
+	"TRACING_CONFIG_STACKDRIVER_PROJECT_ID": "",
+	"TRACING_CONFIG_ZIPKIN_ENDPOINT":        "",
+	"USER_PORT":                             strconv.Itoa(v1.DefaultUserPort),
 }
 
 func probeJSON(container *corev1.Container) string {
@@ -832,3 +939,7 @@ func sortEnv(envs []corev1.EnvVar) {
 		return envs[i].Name < envs[j].Name
 	})
 }
+
+func resourcePtr(q resource.Quantity) *resource.Quantity {
+	return &q
+}
