@@ -19,7 +19,6 @@ package service
 import (
 	"context"
 	"errors"
-	"fmt"
 	"testing"
 
 	// Install our fake informers
@@ -34,8 +33,11 @@ import (
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/logging"
 	"knative.dev/pkg/ptr"
+	"knative.dev/pkg/system"
+	cfgmap "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
+	autoscalercfg "knative.dev/serving/pkg/autoscaler/config"
 	servingclient "knative.dev/serving/pkg/client/injection/client"
 	ksvcreconciler "knative.dev/serving/pkg/client/injection/reconciler/serving/v1/service"
 	configresources "knative.dev/serving/pkg/reconciler/configuration/resources"
@@ -45,6 +47,7 @@ import (
 	apierrs "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/util/clock"
 	clientgotesting "k8s.io/client-go/testing"
 
 	. "knative.dev/pkg/reconciler/testing"
@@ -70,10 +73,10 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "inline - byo rev name used in traffic serialize, with retry",
 		Objects: []runtime.Object{
-			DefaultService("byo-rev", "foo", WithNamedRevision),
+			DefaultService("byo-rev", "foo", WithNamedRevision, WithServiceGeneration(1)),
 			config("byo-rev", "foo",
 				WithNamedRevision,
-				WithGeneration(2)),
+				WithConfigGeneration(2)),
 		},
 		// Route should not be created until config progresses
 		WantCreates: []runtime.Object{},
@@ -81,11 +84,13 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("byo-rev", "foo", WithNamedRevision,
 				// Route conditions should be at init state while Config should be OutOfDate
-				WithInitSvcConditions, WithOutOfDateConfig),
+				WithInitSvcConditions, WithOutOfDateConfig,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}, {
 			Object: DefaultService("byo-rev", "foo", WithNamedRevision,
 				// Route conditions should be at init state while Config should be OutOfDate
-				WithInitSvcConditions, WithOutOfDateConfig),
+				WithInitSvcConditions, WithOutOfDateConfig,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WithReactors: []clientgotesting.ReactionFunc{
 			func(action clientgotesting.Action) (handled bool, ret runtime.Object, err error) {
@@ -99,10 +104,9 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "inline - byo rev name - existing revision - same spec",
 		Objects: []runtime.Object{
-			DefaultService("byo-rev", "foo", WithNamedRevision,
-				WithServiceGeneration(2), WithServiceObservedGeneration),
+			DefaultService("byo-rev", "foo", WithNamedRevision, WithServiceGeneration(2)),
 			config("byo-rev", "foo", WithNamedRevision,
-				WithGeneration(2), WithObservedGen),
+				WithConfigGeneration(2), WithConfigObservedGen),
 			route("byo-rev", "foo", WithNamedRevision,
 				WithRouteGeneration(2), WithRouteObservedGeneration),
 			&v1.Revision{
@@ -125,33 +129,33 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("byo-rev", "foo", WithNamedRevision,
 				// The first reconciliation will initialize the status conditions.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(2), WithServiceObservedGeneration),
 		}},
 	}, {
 		Name: "inline - byo rev name - existing older revision with same spec",
 		Objects: []runtime.Object{
-			DefaultService("byo-rev", "foo", WithNamedRevision,
-				WithServiceGeneration(2), WithServiceObservedGeneration),
+			DefaultService("byo-rev", "foo", WithNamedRevision, WithServiceGeneration(2)),
 			config("byo-rev", "foo", WithNamedRevision,
-				WithGeneration(2), WithObservedGen),
+				WithConfigGeneration(2), WithConfigObservedGen),
 			route("byo-rev", "foo", WithNamedRevision,
 				WithRouteGeneration(2), WithRouteObservedGeneration),
 			rev("byo-rev", "foo", WithNamedRevision,
 				// Older Revision
-				WithGeneration(1), WithObservedGen),
+				WithConfigGeneration(1), WithConfigObservedGen),
 		},
 		Key: "foo/byo-rev",
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("byo-rev", "foo",
-				WithNamedRevision, WithInitSvcConditions),
+				WithNamedRevision, WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(2), WithServiceObservedGeneration),
 		}},
 	}, {
 		Name: "inline - byo rev name - existing older revision with different spec",
 		Objects: []runtime.Object{
-			DefaultService("byo-rev", "foo", WithNamedRevision,
-				WithServiceGeneration(2), WithServiceObservedGeneration),
+			DefaultService("byo-rev", "foo", WithNamedRevision, WithServiceGeneration(2)),
 			config("byo-rev", "foo", WithNamedRevision,
-				WithGeneration(2), WithObservedGen),
+				WithConfigGeneration(2), WithConfigObservedGen),
 			route("byo-rev", "foo", WithNamedRevision,
 				WithRouteGeneration(2), WithRouteObservedGeneration),
 
@@ -174,12 +178,14 @@ func TestReconcile(t *testing.T) {
 		Key: "foo/byo-rev",
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("byo-rev", "foo", WithNamedRevision,
-				WithInitSvcConditions, MarkRevisionNameTaken),
+				WithInitSvcConditions, MarkRevisionNameTaken,
+				WithServiceGeneration(2), WithServiceObservedGeneration,
+			),
 		}},
 	}, {
 		Name: "inline - byo rev name used in traffic",
 		Objects: []runtime.Object{
-			DefaultService("byo-rev", "foo", WithNamedRevision),
+			DefaultService("byo-rev", "foo", WithNamedRevision, WithServiceGeneration(1)),
 		},
 		Key: "foo/byo-rev",
 		WantCreates: []runtime.Object{
@@ -189,7 +195,8 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("byo-rev", "foo", WithNamedRevision,
 				// The first reconciliation will initialize the status conditions.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Created", "Created Configuration %q", "byo-rev"),
@@ -198,7 +205,7 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "create route and configuration",
 		Objects: []runtime.Object{
-			DefaultService("run-latest", "foo", WithRunLatestRollout),
+			DefaultService("run-latest", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 		},
 		Key: "foo/run-latest",
 		WantCreates: []runtime.Object{
@@ -208,7 +215,8 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("run-latest", "foo", WithRunLatestRollout,
 				// The first reconciliation will initialize the status conditions.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Created", "Created Configuration %q", "run-latest"),
@@ -217,7 +225,8 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "steady state",
 		Objects: []runtime.Object{
-			DefaultService("no-updates", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("no-updates", "foo", WithRunLatestRollout, WithInitSvcConditions,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 			route("no-updates", "foo", WithRunLatestRollout),
 			config("no-updates", "foo", WithRunLatestRollout),
 		},
@@ -271,7 +280,8 @@ func TestReconcile(t *testing.T) {
 	}, {
 		Name: "update route and configuration",
 		Objects: []runtime.Object{
-			DefaultService("update-route-and-config", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("update-route-and-config", "foo", WithRunLatestRollout,
+				WithInitSvcConditions),
 			// Mutate the Config/Route to have a different body than we want.
 			config("update-route-and-config", "foo", WithRunLatestRollout,
 				// This is just an unexpected mutation of the config spec vs. the service spec.
@@ -289,7 +299,7 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			DefaultService("update-route-and-config", "foo", WithRunLatestRollout, func(svc *v1.Service) {
 				svc.Spec.GetTemplate().Name = "update-route-and-config-blah"
-			}, WithInitSvcConditions),
+			}, WithInitSvcConditions, WithServiceGeneration(1)),
 			// Mutate the Config/Route to have a different body than we want.
 			config("update-route-and-config", "foo", WithRunLatestRollout,
 				// Change the concurrency to ensure it is corrected.
@@ -313,15 +323,17 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("update-route-and-config", "foo", WithRunLatestRollout, func(svc *v1.Service) {
 				svc.Spec.GetTemplate().Name = "update-route-and-config-blah"
-			}, WithInitSvcConditions, func(svc *v1.Service) {
-				svc.Status.MarkRevisionNameTaken("update-route-and-config-blah")
-			}),
+			}, WithInitSvcConditions, WithServiceGeneration(1), WithServiceObservedGeneration,
+				func(svc *v1.Service) {
+					svc.Status.MarkRevisionNameTaken("update-route-and-config-blah")
+				}),
 		}},
 	}, {
 		Name: "update route and configuration labels",
 		Objects: []runtime.Object{
 			// Mutate the Service to add some more labels
-			DefaultService("update-route-and-config-labels", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceLabel("new-label", "new-value")),
+			DefaultService("update-route-and-config-labels", "foo", WithRunLatestRollout,
+				WithInitSvcConditions, WithServiceLabel("new-label", "new-value")),
 			config("update-route-and-config-labels", "foo", WithRunLatestRollout),
 			route("update-route-and-config-labels", "foo", WithRunLatestRollout),
 		},
@@ -338,14 +350,12 @@ func TestReconcile(t *testing.T) {
 			// Mutate the Service to add some more labels
 			DefaultService("update-child-labels-ignore-route-label", "foo",
 				WithRunLatestRollout, WithInitSvcConditions, WithServiceLabel("new-label", "new-value")),
-			config("update-child-labels-ignore-route-label", "foo",
-				WithRunLatestRollout, WithConfigLabel("serving.knative.dev/route", "update-child-labels-ignore-route-label")),
+			config("update-child-labels-ignore-route-label", "foo", WithRunLatestRollout),
 			route("update-child-labels-ignore-route-label", "foo", WithRunLatestRollout),
 		},
 		Key: "foo/update-child-labels-ignore-route-label",
 		WantUpdates: []clientgotesting.UpdateActionImpl{{
-			Object: config("update-child-labels-ignore-route-label", "foo", WithRunLatestRollout, WithConfigLabel("new-label", "new-value"),
-				WithConfigLabel("serving.knative.dev/route", "update-child-labels-ignore-route-label")),
+			Object: config("update-child-labels-ignore-route-label", "foo", WithRunLatestRollout, WithConfigLabel("new-label", "new-value")),
 		}, {
 			Object: route("update-child-labels-ignore-route-label", "foo", WithRunLatestRollout, WithRouteLabel(map[string]string{"new-label": "new-value",
 				"serving.knative.dev/service": "update-child-labels-ignore-route-label"})),
@@ -382,7 +392,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "routes"),
 		},
 		Objects: []runtime.Object{
-			DefaultService("create-route-failure", "foo", WithRunLatestRollout),
+			DefaultService("create-route-failure", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 		},
 		Key: "foo/create-route-failure",
 		WantCreates: []runtime.Object{
@@ -392,7 +402,8 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("create-route-failure", "foo", WithRunLatestRollout,
 				// First reconcile initializes conditions.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Created", "Created Configuration %q", "create-route-failure"),
@@ -408,7 +419,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("create", "configurations"),
 		},
 		Objects: []runtime.Object{
-			DefaultService("create-config-failure", "foo", WithRunLatestRollout),
+			DefaultService("create-config-failure", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 		},
 		Key: "foo/create-config-failure",
 		WantCreates: []runtime.Object{
@@ -418,7 +429,8 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("create-config-failure", "foo", WithRunLatestRollout,
 				// First reconcile initializes conditions.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "CreationFailed", "Failed to create Configuration %q: %v",
@@ -433,7 +445,8 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("update", "routes"),
 		},
 		Objects: []runtime.Object{
-			DefaultService("update-route-failure", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("update-route-failure", "foo", WithRunLatestRollout,
+				WithInitSvcConditions),
 			// Mutate the Route to have an unexpected body to trigger an update.
 			route("update-route-failure", "foo", WithRunLatestRollout, MutateRoute),
 			config("update-route-failure", "foo", WithRunLatestRollout),
@@ -453,7 +466,8 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("update", "configurations"),
 		},
 		Objects: []runtime.Object{
-			DefaultService("update-config-failure", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("update-config-failure", "foo", WithRunLatestRollout,
+				WithInitSvcConditions, WithServiceGeneration(1), WithServiceObservedGeneration),
 			route("update-config-failure", "foo", WithRunLatestRollout),
 			// Mutate the Config to have an unexpected body to trigger an update.
 			config("update-config-failure", "foo", WithRunLatestRollout,
@@ -475,7 +489,7 @@ func TestReconcile(t *testing.T) {
 			InduceFailure("update", "services"),
 		},
 		Objects: []runtime.Object{
-			DefaultService("run-latest", "foo", WithRunLatestRollout),
+			DefaultService("run-latest", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 		},
 		Key: "foo/run-latest",
 		WantCreates: []runtime.Object{
@@ -486,7 +500,8 @@ func TestReconcile(t *testing.T) {
 			Object: DefaultService("run-latest", "foo", WithRunLatestRollout,
 				// We attempt to update the Service to initialize its
 				// conditions, which is where we induce the failure.
-				WithInitSvcConditions),
+				WithInitSvcConditions, WithServiceObservedGenFailure,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "Created", "Created Configuration %q", "run-latest"),
@@ -498,7 +513,7 @@ func TestReconcile(t *testing.T) {
 		Name: "route and config ready, propagate ready",
 		// When both route and config are ready, the service should become ready.
 		Objects: []runtime.Object{
-			DefaultService("all-ready", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("all-ready", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceGeneration(1)),
 			route("all-ready", "foo", WithRunLatestRollout, RouteReady,
 				WithURL, WithAddress, WithInitRouteConditions,
 				WithStatusTraffic(
@@ -507,7 +522,7 @@ func TestReconcile(t *testing.T) {
 						Percent:      ptr.Int64(100),
 					}), MarkTrafficAssigned, MarkIngressReady),
 			config("all-ready", "foo", WithRunLatestRollout,
-				WithGeneration(1), WithObservedGen,
+				WithConfigGeneration(1), WithConfigObservedGen,
 				// These turn a Configuration to Ready=true
 				WithLatestCreated("all-ready-00001"), WithLatestReady("all-ready-00001")),
 		},
@@ -527,7 +542,7 @@ func TestReconcile(t *testing.T) {
 		// When both route and config are ready, the service should become ready.
 		Objects: []runtime.Object{
 			DefaultService("all-ready", "foo", WithRunLatestRollout, WithInitSvcConditions,
-				WithReadyConfig("all-ready-00001")),
+				WithReadyConfig("all-ready-00001"), WithServiceGeneration(1)),
 			route("all-ready", "foo", WithRunLatestRollout, RouteReady,
 				WithURL, WithAddress, WithInitRouteConditions,
 				WithStatusTraffic(v1.TrafficTarget{
@@ -535,7 +550,7 @@ func TestReconcile(t *testing.T) {
 					Percent:      ptr.Int64(100),
 				}), MarkTrafficAssigned, MarkIngressReady),
 			config("all-ready", "foo", WithRunLatestRollout,
-				WithGeneration(1), WithObservedGen, WithGeneration(2),
+				WithConfigGeneration(1), WithConfigObservedGen, WithConfigGeneration(2),
 				// These turn a Configuration to Ready=true
 				WithLatestCreated("all-ready-00001"), WithLatestReady("all-ready-00001")),
 		},
@@ -556,7 +571,7 @@ func TestReconcile(t *testing.T) {
 		// When both route and config are ready, but the route points to the previous revision
 		// the service should not be ready.
 		Objects: []runtime.Object{
-			DefaultService("config-only-ready", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("config-only-ready", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceGeneration(1)),
 			route("config-only-ready", "foo", WithRunLatestRollout, RouteReady,
 				WithURL, WithAddress, WithInitRouteConditions,
 				WithStatusTraffic(v1.TrafficTarget{
@@ -564,7 +579,7 @@ func TestReconcile(t *testing.T) {
 					Percent:      ptr.Int64(100),
 				}), MarkTrafficAssigned, MarkIngressReady),
 			config("config-only-ready", "foo", WithRunLatestRollout,
-				WithGeneration(2 /*will generate revision -00002*/), WithObservedGen,
+				WithConfigGeneration(2 /*will generate revision -00002*/), WithConfigObservedGen,
 				// These turn a Configuration to Ready=true
 				WithLatestCreated("config-only-ready-00002"), WithLatestReady("config-only-ready-00002")),
 		},
@@ -584,16 +599,16 @@ func TestReconcile(t *testing.T) {
 		// Gen 2: config update fails;
 		//    => service is still OK serving Gen 1.
 		Objects: []runtime.Object{
-			DefaultService("config-fails", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("config-fails", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceGeneration(1)),
 			route("config-fails", "foo", WithRunLatestRollout, RouteReady,
 				WithURL, WithAddress, WithInitRouteConditions,
 				WithStatusTraffic(v1.TrafficTarget{
 					RevisionName: "config-fails-00001",
 					Percent:      ptr.Int64(100),
 				}), MarkTrafficAssigned, MarkIngressReady),
-			config("config-fails", "foo", WithRunLatestRollout, WithGeneration(2),
+			config("config-fails", "foo", WithRunLatestRollout, WithConfigGeneration(2),
 				WithLatestReady("config-fails-00001"), WithLatestCreated("config-fails-00002"),
-				MarkLatestCreatedFailed("blah"), WithObservedGen),
+				MarkLatestCreatedFailed("blah"), WithConfigObservedGen),
 		},
 		Key: "foo/config-fails",
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
@@ -610,9 +625,9 @@ func TestReconcile(t *testing.T) {
 		Name: "configuration failure is propagated",
 		// When config fails, the service should fail.
 		Objects: []runtime.Object{
-			DefaultService("config-fails", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("config-fails", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceGeneration(1)),
 			route("config-fails", "foo", WithRunLatestRollout, RouteReady),
-			config("config-fails", "foo", WithRunLatestRollout, WithGeneration(1), WithObservedGen,
+			config("config-fails", "foo", WithRunLatestRollout, WithConfigGeneration(1), WithConfigObservedGen,
 				WithLatestCreated("config-fails-00001"), MarkLatestCreatedFailed("blah")),
 		},
 		Key: "foo/config-fails",
@@ -625,10 +640,10 @@ func TestReconcile(t *testing.T) {
 		Name: "route failure is propagated",
 		// When route fails, the service should fail.
 		Objects: []runtime.Object{
-			DefaultService("route-fails", "foo", WithRunLatestRollout, WithInitSvcConditions),
+			DefaultService("route-fails", "foo", WithRunLatestRollout, WithInitSvcConditions, WithServiceGeneration(1)),
 			route("route-fails", "foo", WithRunLatestRollout,
 				RouteFailed("Propagate me, please", "")),
-			config("route-fails", "foo", WithRunLatestRollout, WithGeneration(1), WithObservedGen,
+			config("route-fails", "foo", WithRunLatestRollout, WithConfigGeneration(1), WithConfigObservedGen,
 				// These turn a Configuration to Ready=true
 				WithLatestCreated("route-fails-00001"), WithLatestReady("route-fails-00001")),
 		},
@@ -644,14 +659,15 @@ func TestReconcile(t *testing.T) {
 		Name:    "existing configuration without an owner causes a failure",
 		WantErr: true,
 		Objects: []runtime.Object{
-			DefaultService("run-latest", "foo", WithRunLatestRollout),
+			DefaultService("run-latest", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 			config("run-latest", "foo", WithRunLatestRollout, WithConfigOwnersRemoved),
 		},
 		Key: "foo/run-latest",
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("run-latest", "foo", WithRunLatestRollout,
 				// The first reconciliation will initialize the status conditions.
-				WithInitSvcConditions, MarkConfigurationNotOwned),
+				WithInitSvcConditions, MarkConfigurationNotOwned,
+				WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError", `service: "run-latest" does not own configuration: "run-latest"`),
@@ -660,7 +676,7 @@ func TestReconcile(t *testing.T) {
 		Name:    "existing route without an owner causes a failure",
 		WantErr: true,
 		Objects: []runtime.Object{
-			DefaultService("run-latest", "foo", WithRunLatestRollout),
+			DefaultService("run-latest", "foo", WithRunLatestRollout, WithServiceGeneration(1)),
 			config("run-latest", "foo", WithRunLatestRollout),
 			route("run-latest", "foo", WithRunLatestRollout, WithRouteOwnersRemoved),
 		},
@@ -668,7 +684,7 @@ func TestReconcile(t *testing.T) {
 		WantStatusUpdates: []clientgotesting.UpdateActionImpl{{
 			Object: DefaultService("run-latest", "foo", WithRunLatestRollout,
 				// The first reconciliation will initialize the status conditions.
-				WithInitSvcConditions, MarkRouteNotOwned),
+				WithInitSvcConditions, MarkRouteNotOwned, WithServiceGeneration(1), WithServiceObservedGeneration),
 		}},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError", `service: "run-latest" does not own route: "run-latest"`),
@@ -688,7 +704,7 @@ func TestReconcile(t *testing.T) {
 					RevisionName: "new-owner-00001",
 					Percent:      ptr.Int64(100),
 				}), MarkTrafficAssigned, MarkIngressReady),
-			config("new-owner", "foo", WithRunLatestRollout, WithGeneration(1), WithObservedGen,
+			config("new-owner", "foo", WithRunLatestRollout, WithConfigGeneration(1), WithConfigObservedGen,
 				// These turn a Configuration to Ready=true
 				WithLatestCreated("new-owner-00001"), WithLatestReady("new-owner-00001")),
 		},
@@ -774,7 +790,27 @@ func TestReconcile(t *testing.T) {
 func TestNew(t *testing.T) {
 	ctx, _ := SetupFakeContext(t)
 
-	c := NewController(ctx, configmap.NewStaticWatcher())
+	configMapWatcher := configmap.NewStaticWatcher(&corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cfgmap.FeaturesConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	}, &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cfgmap.DefaultsConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	}, &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      autoscalercfg.ConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	})
+
+	c := NewController(ctx, configMapWatcher)
 
 	if c == nil {
 		t.Fatal("Expected NewController to return a non-nil value")
@@ -784,10 +820,7 @@ func TestNew(t *testing.T) {
 func config(name, namespace string, so ServiceOption, co ...ConfigOption) *v1.Configuration {
 	s := DefaultService(name, namespace, so)
 	s.SetDefaults(context.Background())
-	cfg, err := resources.MakeConfiguration(s)
-	if err != nil {
-		panic(fmt.Sprint("MakeConfiguration() = ", err))
-	}
+	cfg := resources.MakeConfiguration(s)
 	for _, opt := range co {
 		opt(cfg)
 	}
@@ -797,10 +830,7 @@ func config(name, namespace string, so ServiceOption, co ...ConfigOption) *v1.Co
 func route(name, namespace string, so ServiceOption, ro ...RouteOption) *v1.Route {
 	s := DefaultService(name, namespace, so)
 	s.SetDefaults(context.Background())
-	route, err := resources.MakeRoute(s)
-	if err != nil {
-		panic(fmt.Sprint("MakeRoute() = ", err))
-	}
+	route := resources.MakeRoute(s)
 	for _, opt := range ro {
 		opt(route)
 	}
@@ -840,5 +870,5 @@ func RouteFailed(reason, message string) RouteOption {
 
 func rev(name, namespace string, so ServiceOption, co ...ConfigOption) *v1.Revision {
 	cfg := config(name, namespace, so, co...)
-	return configresources.MakeRevision(cfg)
+	return configresources.MakeRevision(context.Background(), cfg, clock.RealClock{})
 }
